
# q-expansion of newform 592.2.w.b, downloaded from the LMFDB on 08 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 592
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[223, [1, 0]], [149, [1, 0]], [113, [0, 1]]]
aps_data = [[0, 0], [0, 1], [2, -1], [0, 1], [0, 0], [2, -1], [1, 1], [2, -1], [2, -4], [-4, 8], [2, -4], [-3, -4], [0, 9], [-2, 4], [0, 0], [-3, 3], [-1, -1], [2, -1], [0, 13], [0, -15], [-2, 0], [-14, 7], [-9, 9], [-3, -3], [4, -8], [-6, 0], [10, -20], [0, 9], [-7, -7], [1, 1], [11, -11], [-9, -9], [-18, 0], [-5, 5], [-6, 0], [0, -19], [5, -5], [11, 11], [-14, 7], [-15, 15], [-10, 20], [0, 5], [-6, 12], [12, -24], [21, -21], [-6, 12], [4, 0], [-8, 0], [-22, 11], [0, 13], [6, 0], [7, 7], [34, -17], [10, -20], [-3, -3], [0, 21], [-30, 0], [7, -7], [18, -9], [9, 9], [34, -17], [0, 9], [16, 0], [15, 15], [13, 13], [-15, 15], [3, 3], [0, 13], [-10, 20], [-19, 19], [5, 5], [12, 0], [0, -7], [0, -19], [7, -7], [10, -5], [18, -9], [-2, 0], [12, -24], [1, 1], [-33, 33], [8, -16], [-14, 7], [-2, 0], [-38, 19], [-12, 0], [-22, 11], [18, -9], [1, 1], [-6, 3], [22, -44], [7, 7], [-14, 28], [-12, 0], [-6, 3], [-9, -9], [-39, 39], [0, -15], [2, -1], [8, -16], [22, -44], [5, 5], [18, -36], [-20, 40], [-5, 5], [-19, -19], [10, -5], [30, 0], [0, -3], [5, -5], [-5, -5], [-11, 11], [-15, 15], [-44, 0], [15, 15], [-3, 3], [-10, 20], [26, -13], [42, -21], [0, -27], [26, -13], [0, -19], [42, 0], [-17, -17], [-17, 17], [13, 13], [-4, 8], [-21, 21], [-38, 19], [0, -23], [4, 0], [0, -15], [20, 0], [-3, -3], [45, -45], [12, -24], [0, -15], [-32, 0], [21, 21], [-22, 11], [0, 49], [0, -3], [47, -47], [34, -17], [21, 21], [-9, 9], [25, 25], [-28, 56], [6, -12], [39, -39], [-2, 0], [0, -3], [58, -29], [48, 0], [42, -21], [34, -68], [-2, 4], [0, -39], [-59, 59], [45, -45], [15, 15], [9, -9], [-30, 15], [3, -3], [25, 25], [3, -3], [-22, 44], [21, 21]]
